##PROCEDURE(help) ResChain[ResChainBranching]
##HALFLINE This procedure eliminates requested variables from a system of polynomial equations.
##AUTHOR AmirHosein Sadeghimanesh
##DATE November 2021
##CALLINGSEQUENCE
##- ResChainBranching( 'F', 'vars' )
##PARAMETERS
##- 'F'                  : a list of polynomials.
##- 'vars'               : a list of names.
##RETURNS
##- It returns a list of polynomials after the last step of resultants where all the requested variables are eliminated. Note that if the algorithm returns a list of only one element, 0, it means either over the complex space the projection is the whole parameter space (corresponding to the indeterminants that are not asked to be eliminated from the system), or the algorithm hits line 11 of Algorithm 1 in the paper "Resultant Tools for Parametric Polynomial Systems with Application to Population Models, AmirHosein Sadeghimanesh, Matthew England, 2022".
##DESCRIPTION
##- It is an alternative to the elimination via Grobner basis or Dixon resultant that works for n variables and m polynomials. It may produce extra components, but certainly contains the componets obtained by the other two approaches. For some examples where the two others fail or take a long time to finish, this alternative method may be helpful.
##EXAMPLES
##> with(ResChain):
##> F := [(x^2+y^2-a)*(y-x^2), (x+y)*(x*y-b), (x^3+a*y^2+b)]:
##> vars := [y, x]:
##> parameters_conditions := ResChainBranching(F, vars);
##- parameters_conditions := [-a^4*b^4 + a^7 - 5*a^5*b^2 - 2*a^4*b^3 + 5*a^3*b^4 + 2*a^5*b - a^4*b^2 - 8*a^3*b^3 - 2*a^2*b^4 + 4*a*b^5 - b^6 + a^3*b^2 - 2*a^2*b^3 - 3*a*b^4 - b^4, a^4 - 1/2*a^3 + 4*a^2*b + 4*b^2, a, a^3*b + 8, b, a + b - 1, a + b + 1]
##SEEALSO
##- ResChain:-ResChainSimple
##- PolynomialIdeals:-EliminationIdeal
##XERFMAP
##- 

ResChainBranching := proc(F :: list(polynom), vars :: list(symbol)) :: list;

	description "It eliminates the variables given in the second argument via a chain of resultants. It returns a list of polynomials after finishing the elimination.";

	local
		n :: posint,
		m :: posint,
		polynomial :: polynom,
		variable :: symbol,
		ResChainQueueOld :: Array(Array),
		ResChainQueueNew :: Array(Array),
		ResChainBranches :: Array(Array),
		i :: posint,
		resChainOld :: Array,
		resChainNew :: Array,
		level :: nonnegint,
		var :: name,
		i1 :: posint,
		i2 :: posint,
        i3 :: posint,
		f :: polynom,
		g :: polynom,
		j :: posint,
		res :: polynom,
		res_factors :: Array(polynom),
        old_branch_counter :: nonnegint,
		new_branch_counter :: nonnegint,
		branch :: Array,
		element :: Array,
		output :: Array(polynom);

	n := numelems(vars);
	m := numelems(F);
	
	if m = 1 and 0 in F then
		userinfo(1, ResChainBranching, 'NoName', printf("The input list of polynomials was just 0."));
		return [0]: # the projection is everything.
	end if:

	for polynomial in F do
		if numelems(indets(polynomial)) = 0 then
			userinfo(1, ResChainBranching, 'NoName', printf("The input list of polynomials contains a constant polynomial."));
			return [1]: # the projection is nothing.
		end if:
	end do:

	if m = 1 then
		# because the proc hasn't stopped by the previous return line, then it means this polynomial is not a constant polynomial.
		userinfo(1, ResChainBranching, 'NoName', printf("There is only one polynomial and therfore no chain of resultant is applicable."));
		for variable in vars do
			if variable in indets(F[1]) then
				userinfo(1, ResChainBranching, 'NoName', printf("The only polynomial of the input list contains one of the variables that are requested to be eliminated."));
				return([0]): # at least in complex space, the projection is everything.
			end if:
		end do:
		# if not exited so far, it means that it does not have any of the variables in vars.
		userinfo(1, ResChainBranching, 'NoName', printf("The only polynomial of the input list does not contain any of the variables that are requested to be eliminated."));
		return(F); # the projection is given by the input list itself.
	end if:

    ResChainQueueNew := Array([]): 
	# to store the inputs for single step of ResChainBranching.

	ArrayTools:-Append(ResChainQueueNew, Array([])):
	# only one branch at the top at step 0. Because here we only do factorization for preparation of the first step.
	ArrayTools:-Extend(ResChainQueueNew[1], [seq(Array([]), i=1..m)]):

	for i from 1 by 1 to m do
		ArrayTools:-Extend(ResChainQueueNew[1][i], Factorize(F[i])): 
        # read as ResChainSpecificLevel[ branch ][ polynomial factors array ][ individual factor ]
	end do:

    ResChainBranches := Array([]):
	# to store the end results of each final branch.

    #####

	for level from n-1 by -1 to 0 do

        ResChainQueueOld := Array([ seq( branch, branch in ResChainQueueNew ) ]): # here a branch means the same as a branch.
        ResChainQueueNew := Array([]):
	    # to store the result of calls of the old queue. New branches of calls may get generated.
		new_branch_counter := 0: 
		# counts the number of the branches in the next step.

		var := vars[level + 1]:
		userinfo(1, ResChainBranching, 'NoName', printf("leading variable = %a\n", var));

        
        old_branch_counter := 0:
        # This variable is not needed, it is just to produce some userinfo messages to keep the user updated that the algorithm is moving along the old branches.

        while numelems(ResChainQueueOld) <> 0 do

            resChainOld := ResChainQueueOld[-1]:
            ArrayTools:-Remove(ResChainQueueOld, -1):
            old_branch_counter := old_branch_counter + 1:
            m := numelems(resChainOld):

            if m = 1 and 0 in resChainOld[1] then
                userinfo(1, ResChainBranching, 'NoName', printf("Algorithm encountered a 0 branch which its solution set union with the other branches' is everything. Therefore the algorithm stops in here and returns 0."));
                return [0]: # the projection is everything.
            end if:

            for i from 1 by 1 to numelems(resChainOld) do
                if numelems(indets(resChainOld[i])) = 0 then
                    userinfo(2, ResChainBranching, 'NoName', printf("A branch with empty solution set is detected and disregarded."));
                    # the projection of this branch is empty and has no role in the final union.
                end if:
            end do:

            if m = 1 then
                # because the proc hasn't stopped by the previous return line, then it means this polynomial is not a constant polynomial.
                userinfo(2, ResChainBranching, 'NoName', printf("A branch with only one polynomial is detected. No chain of resultant is applicable on this branch.\n"));
                #printf("The single polynomial branch is\n%a\n", resChainOld); # extra printf for a check-point.
                for variable in vars do
                    if variable in indets(resChainOld[1]) then
                        userinfo(1, ResChainBranching, 'NoName', printf("The only polynomial of this branch contains one of the variables that are requested to be eliminated. The projection of this branch, at least in the complex space, is everything, therefore a 0-branch is detected and since union of its solution set with the other branches' is everything, the algorithm stops here and resturns 0."));
                        return([0]): # at least in complex space, the projection is everything.
                    end if:
                end do:
                # if not exited so far, it means that it does not have any of the variables in vars.
                userinfo(2, ResChainBranching, 'NoName', printf("The only polynomial of this branch does not contain any of the variables that are requested to be eliminated. Therefore this branch is pushed to the final branches."));
                ArrayTools:-Append(ResChainBranches, resChainOld[1]) # the projection of this branch is itself.
            end if:

            #####

            new_branch_counter := 0: 
            # counts the number of the branches in the next step. This is also not a necessary variable.

            userinfo(3, ResChainBranching, 'NoName', printf("old branch no. %d is under consideration.\n", old_branch_counter)):

            for i1 from 1 by 1 to numelems(resChainOld[1]) do

                userinfo(3, ResChainBranching, 'NoName', printf("i1 = %d\n", i1)):
                # a new branch for the next step is created.
                resChainNew := Array([]):
                new_branch_counter := new_branch_counter + 1:
                userinfo(3, ResChainBranching, 'NoName', printf("new branch no. %d is created for the next level.\n", new_branch_counter)):
                ArrayTools:-Extend(resChainNew, [seq(Array([]), i=1..m-1)]):
                # end of initialization of a new branch.
                f := resChainOld[1][i1]:

                if not var in indets(f) then

                    for i from 1 by 1 to m - 1 do

                        userinfo(3, ResChainBranching, 'NoName', printf("i = %d\n", i)):
                        ArrayAddIfNotIn(resChainNew[i], f):

                        for i2 from 1 by 1 to numelems(resChainOld[i + 1]) do

                            userinfo(3, ResChainBranching, 'NoName', printf("i2 = %d\n", i2)):
                            g := resChainOld[i + 1][i2]:

                            if not var in indets(g) then
                                ArrayAddIfNotIn(resChainNew[i], g):
                            end if:

                        end do:

                    end do:

                else

                    for i from 1 by 1 to m - 1 do

                        userinfo(3, ResChainBranching, 'NoName', printf("i = %d\n", i)):

                        for i2 from 1 by 1 to numelems(resChainOld[i + 1]) do

                            userinfo(3, ResChainBranching, 'NoName', printf("i2 = %d\n", i2)):
                            g := resChainOld[i + 1][i2]:

                            if not var in indets(g) then

                                ArrayAddIfNotIn(resChainNew[i], g):

                            else

                                if expand(f - g) = 0 then

                                    userinfo(2, ResChainBranching, 'NoName', printf("When trying to eliminate %a, resultant of two equal polynomials is asked. This means two polynomials of the previous level have a common factor. Therefore a new branch for the same level is created and added to the end of the queue.\n",var));
                                    ArrayTools:-Append(ResChainQueueOld, Array([ Array([f]) ])): 
                                    # creating the new branch containing f as a single-factor polynomial standing instead of at least the first two former polynomials.
                                    for i3 from 2 by 1 to m do
                                        if not f in resChainOld[i3] then
                                            ArrayTools:-Append(ResChainQueueOld[-1], resChainOld[i3]):
                                        end if:
                                    end do:

                                else:

                                    res := resultant(f, g, var):
                                    res_factors := Factorize(res):
                                    
                                    for polynomial in res_factors do
                                        userinfo(3, ResChainBranching, 'NoName', printf("polynomial = %a\n", polynomial)):
                                        ArrayAddIfNotIn(resChainNew[i], polynomial):
                                    end do:

                                end if:

                            end if:

                        end do:

                    end do:

                end if:

                ArrayTools:-Append(ResChainQueueNew, Array([seq(element, element in resChainNew)])):

            end do: # new branch finished

        end do: # old branch finished.

	end do: # level finished.


    # every branch in ResChainQueueNew now is a final branch as well.
    for branch in ResChainQueueNew do
        ArrayTools:-Append(ResChainBranches, branch):
    end do:

	output := Array([]):

    for branch in ResChainBranches do
        for element in branch do
            for polynomial in element do
                ArrayAddIfNotIn(output, polynomial):
            end do:
        end do:
    end do:

	return( convert(output, list) ):

end proc: