##PROCEDURE(help) ResChain[ResChainSimple]
##HALFLINE This procedure eliminates requested variables from a system of polynomial equations.
##AUTHOR AmirHosein Sadeghimanesh
##DATE November 2021
##CALLINGSEQUENCE
##- ResChainSimple( 'F', 'vars' )
##PARAMETERS
##- 'F'                  : a list of polynomials.
##- 'vars'               : a list of names.
##RETURNS
##- It returns a list of polynomials after the last step of resultants where all the requested variables are eliminated. Note that if the algorithm returns a list of only one element, 0, it means either over the complex space the projection is the whole parameter space (corresponding to the indeterminants that are not asked to be eliminated from the system), or the algorithm hits line 11 of Algorithm 1 in the paper "Resultant Tools for Parametric Polynomial Systems with Application to Population Models, AmirHosein Sadeghimanesh, Matthew England, 2022".
##DESCRIPTION
##- It is an alternative to the elimination via Grobner basis or Dixon resultant that works for n variables and m polynomials. It may produce extra components, but certainly contains the componets obtained by the other two approaches. For some examples where the two others fail or take a long time to finish, this alternative method may be helpful.
##EXAMPLES
##> with(ResChain):
##> F := [(x^2+y^2-a)*(y-x^2), (x+y)*(x*y-b), (x^3+a*y^2+b)]:
##> vars := [y, x]:
##> parameters_conditions := ResChainSimple(F, vars);
##- parameters_conditions := [a, a^3*b + 8, b, -a^6*b^8 + 9*a^5*b^8 - 18*a^4*b^8 + 6*a^3*b^9 - b^12 - 9*a^7*b^4 - 6*a^6*b^5 + 3*a^3*b^8 - 18*a^2*b^9 - 3*a^6*b^4 - 36*a^5*b^5 - 27*a^4*b^6 - 2*a^3*b^7 - 3*b^10 + a^9 - 21*a^3*b^6 - 18*a^2*b^7 - 3*a^6*b^2 - 3*b^8 + 3*a^3*b^4 - b^6, a^3 - b^2, a + b - 1, a + b + 1, a^3 - 4*a^2 - 2*a*b - b^2 + 3*a - 1, a^8*b^4 + 4*a^8*b^3 + 6*a^8*b^2 + 4*a^6*b^4 + 4*a^8*b + 12*a^6*b^3 + a^8 + 12*a^6*b^2 + 6*a^4*b^4 + 4*a^6*b - a^5*b^2 + 12*a^4*b^3 + 6*a^4*b^2 - 8*a^3*b^3 + 4*a^2*b^4 - 6*a^3*b^2 + 4*a^2*b^3 - 4*a^3*b + 8*a*b^3 + b^4 - a^3 + 3*a*b^2 + b^2, -a^4*b^4 + a^7 - 5*a^5*b^2 - 2*a^4*b^3 + 5*a^3*b^4 + 2*a^5*b - a^4*b^2 - 8*a^3*b^3 - 2*a^2*b^4 + 4*a*b^5 - b^6 + a^3*b^2 - 2*a^2*b^3 - 3*a*b^4 - b^4, a^8 + 8*a^6*b + 24*a^4*b^2 + 32*a^2*b^3 + 16*b^4 - 2*a^3, a^4 - 1/2*a^3 + 4*a^2*b + 4*b^2]
##SEEALSO
##- ResChain:-ResChainBranching
##- PolynomialIdeals:-EliminationIdeal
##XERFMAP
##- 

ResChainSimple := proc(F :: list(polynom), vars :: list(symbol)) :: list;

	description "It eliminates the variables given in the second argument via a chain of resultants. It returns a list of polynomials after finishing the elimination.";

	local
		n :: posint,
		m :: posint,
		polynomial :: polynom,
		variable :: symbol,
		ResChainQueueOld :: Array(Array),
		ResChainQueueNew :: Array(Array),
		ResChainStrands :: Array(Array),
		i :: posint,
		resChainOld :: Array,
		resChainNew :: Array,
        strand :: Array,
		level :: nonnegint,
		var :: name,
		i1 :: posint,
		i2 :: posint,
        i3 :: posint,
		f :: polynom,
		g :: polynom,
		j :: posint,
		res :: polynom,
		res_factors :: Array(polynom),
		element :: Array,
		output :: Array(polynom);

	n := numelems(vars);
	m := numelems(F);
	
	if m = 1 and 0 in F then
		userinfo(1, ResChainSimple, 'NoName', printf("The input list of polynomials was just 0."));
		return [0]: # the projection is everything.
	end if:

	for polynomial in F do
		if numelems(indets(polynomial)) = 0 then
			userinfo(1, ResChainSimple, 'NoName', printf("The input list of polynomials contains a constant polynomial."));
			return [1]: # the projection is nothing.
		end if:
	end do:

	if m = 1 then
		# because the proc hasn't stopped by the previous return line, then it means this polynomial is not a constant polynomial.
		userinfo(1, ResChainSimple, 'NoName', printf("There is only one polynomial and therfore no chain of resultant is applicable."));
		for variable in vars do
			if variable in indets(F[1]) then
				userinfo(1, ResChainSimple, 'NoName', printf("The only polynomial of the input list contains one of the variables that are requested to be eliminated."));
				return([0]): # at least in complex space, the projection is everything.
			end if:
		end do:
		# if not exited so far, it means that it does not have any of the variables in vars.
		userinfo(1, ResChainSimple, 'NoName', printf("The only polynomial of the input list does not contain any of the variables that are requested to be eliminated."));
		return(F); # the projection is given by the input list itself.
	end if:

	ResChainQueueNew := Array([ Array([]) ]): 
	# to store the inputs for single step of ResChainSimple. 
	# It contains only one call at the beginning. If no two polynomials share a common factor in any stage, then until the end it goes with a single call.

	ArrayTools:-Extend(ResChainQueueNew[1], [seq(Array([]), i=1..m)]):

	for i from 1 by 1 to m do
		ArrayTools:-Extend(ResChainQueueNew[1][i], Factorize(F[i])):
	end do:

	ResChainStrands := Array([]):
	# to store the end results of each call strands.

    for level from n-1 by -1 to 0 do

        ResChainQueueOld := Array([ seq( strand, strand in ResChainQueueNew ) ]):
        ResChainQueueNew := Array([]):
	    # to store the result of calls of the old queue. New strand of calls may get generated.

        var := vars[level + 1]:
        userinfo(1, ResChainSimple, 'NoName', printf("leading variable = %a\n",var));

        while numelems(ResChainQueueOld) <> 0 do

            resChainOld := ResChainQueueOld[-1]:
            # printf("Current call is on\n%a\n",resChainOld); # extra printf for a check-point.
            ArrayTools:-Remove(ResChainQueueOld, -1):
            m := numelems(resChainOld):

            if m = 1 and 0 in resChainOld[1] then
                userinfo(1, ResChainSimple, 'NoName', printf("Algorithm encountered a 0 strand which its solution set union with the other strands' is everything. Therefore the algorithm stops in here and returns 0."));
                return [0]: # the projection is everything.
            end if:

            for i from 1 by 1 to numelems(resChainOld) do
                if numelems(indets(resChainOld[i])) = 0 then
                    userinfo(2, ResChainSimple, 'NoName', printf("A strand with empty solution set is detected and disregarded."));
                    # the projection of this strand is empty and has no role in the final union.
                end if:
            end do:

            if m = 1 then
                # because the proc hasn't stopped by the previous return line, then it means this polynomial is not a constant polynomial.
                userinfo(2, ResChainSimple, 'NoName', printf("A strand with only one polynomial is detected. No chain of resultant is applicable on this strand.\n"));
                #printf("The single polynomial strand is\n%a\n", resChainOld); # extra printf for a check-point.
                for variable in vars do
                    if variable in indets(resChainOld[1]) then
                        userinfo(1, ResChainSimple, 'NoName', printf("The only polynomial of this strand contains one of the variables that are requested to be eliminated. The projection of this strand, at least in the complex space, is everything, therefore a 0-strand is detected and since union of its solution set with the other strands' is everything, the algorithm stops here and resturns 0."));
                        return([0]): # at least in complex space, the projection is everything.
                    end if:
                end do:
                # if not exited so far, it means that it does not have any of the variables in vars.
                userinfo(2, ResChainSimple, 'NoName', printf("The only polynomial of this strand does not contain any of the variables that are requested to be eliminated. Therefore this strand is pushed to the final strands."));
                ArrayTools:-Append(ResChainStrands, resChainOld[1]) # the projection of this strand is itself.
            end if:

            resChainNew := Array([seq(Array([]), i=1..m-1)]):

            for i from 1 by 1 to m - 1 do

                userinfo(3, ResChainSimple, 'NoName', printf("i = %d\n",i));

                for i1 from 1 by 1 to numelems(resChainOld[1]) do

                    userinfo(3, ResChainSimple, 'NoName', printf("i1 = %d\n",i1));
                    f := resChainOld[1][i1]:

                    if not var in indets(f) then

                        ArrayAddIfNotIn(resChainNew[i], f):

                        for i2 from 1 by 1 to numelems(resChainOld[i + 1]) do

                            userinfo(3, ResChainSimple, 'NoName', printf("i2 = %d\n",i2));
                            g := resChainOld[i + 1][i2]:

                            if not var in indets(g) then
                                ArrayAddIfNotIn(resChainNew[i], g):
                            end if:

                        end do:

                    else

                        for i2 from 1 by 1 to numelems(resChainOld[i + 1]) do

                            userinfo(3, ResChainSimple, 'NoName', printf("i2 = %d\n",i2));
                            g := resChainOld[i + 1][i2]:

                            if not var in indets(g) then

                                ArrayAddIfNotIn(resChainNew[i], g):

                            else

                                if expand(f - g) = 0 then

                                    userinfo(2, ResChainSimple, 'NoName', printf("When trying to eliminate %a, resultant of two equal polynomials is asked. This means two polynomials of the previous level have a common factor. Therefore a new strand for the same level is created and added to the end of the queue.\n",var));
                                    ArrayTools:-Append(ResChainQueueOld, Array([ Array([f]) ])): 
                                    # creating the new strand containing f as a single-factor polynomial standing instead of at least the first two former polynomials.
                                    for i3 from 2 by 1 to m do
                                        if not f in resChainOld[i3] then
                                            ArrayTools:-Append(ResChainQueueOld[-1], resChainOld[i3]):
                                        end if:
                                    end do:

                                else:

                                    res := resultant(f, g, var):
                                    res_factors := Factorize(res):
                                    
                                    for polynomial in res_factors do
                                        userinfo(3,ResChainSimple,'NoName',printf("polynomial = %a\n",polynomial));
                                        ArrayAddIfNotIn(resChainNew[i], polynomial):
                                    end do:

                                end if:

                            end if:

                        end do:

                    end if:

                end do:

            end do:

            ArrayTools:-Append(ResChainQueueNew, Array([seq(element, element in resChainNew)])):

        end do: # call finished.

	end do: # level finished.

    # every strand in ResChainQueueNew now is a final strand as well.
    for strand in ResChainQueueNew do
        ArrayTools:-Append(ResChainStrands, strand):
    end do:

	output := Array([]):

    for strand in ResChainStrands do
        for element in strand do
            for polynomial in element do
                ArrayAddIfNotIn(output, polynomial):
            end do:
        end do:
    end do:

	return( convert(output, list) ):

end proc: