##PROCEDURE(help) ResChain[Factorize]
##HALFLINE This procedure calls Maple\'s 'factors', but rescales the coefficients if asked. It is taken from DEWCAD package.
##AUTHOR AmirHosein Sadeghimanesh
##DATE June 2021
##CALLINGSEQUENCE
##- Factorize( 'f', { 'integerPolynomials' := 'false' } )
##PARAMETERS
##- 'f'                  : A polynomial.
##- 'integerPolynomials' : true/false.
##RETURNS
##- It factorize f and returns the Array of irreducible factors of f. If 'integerPolynomials' is set to 'true', it rescales the factors to have only integer coefficients.
##DESCRIPTION
##- Maple\'s 'factors' and 'sqrfree' return a list of two elements, the first element is a scalar and the second element is the list of pairs of irreducible or square-free factors of f with their powers respectively. In this package we do not need the first element and also the powers of the components. So 'Factorize' ignores these parts. 
##- If the optional keyword argument 'integerPolynomials' is set to 'true', then it rescales the polynomials in the output of 'factors' so that all coefficients are integers. Maple\'s 'sqrfree' keeps all coeficients to be 'integer' while 'factors' on the other hand tries to keep all factors to be monic. 'Factorize' with 'integerPolynomials=ture' uses the commands 'denom' and 'content' of Maple.
##EXAMPLES
##> with(ResChain): # Factorize is a local proc in ResChain by default.
##> f := expand((x-1/24)*(27*x^3+x/2-5/16)*(x^2/7+x+1));
##- f := 27/7*x^6 + 1503/56*x^5 + 1453/56*x^4 - 113/168*x^3 + 151/896*x^2 - 41/128*x + 5/384
##> sqrfree(f);
##- [1/2688, [[10368*x^6 + 72144*x^5 + 69744*x^4 - 1808*x^3 + 453*x^2 - 861*x + 35, 1]]]
##> factors(f);
##- [27/7, [[x - 1/24, 1], [x^3 + 1/54*x - 5/432, 1], [x^2 + 7*x + 7, 1]]]
##> Factorize(f);
##- Array([x - 1/24, x^3 + 1/54*x - 5/432, x^2 + 7*x + 7])
##> Factorize(f,integerPolynomials=true);
##- Array([24*x - 1, 432*x^3 + 8*x - 5, x^2 + 7*x + 7])
##SEEALSO
##- factors
##- sqrfree
##XERFMAP
##- 

Factorize := proc(f :: polynom, {integerPolynomials :: truefalse := false}) :: Array(polynom):

    description "Receives a polynomial, f, and returns an Array containing irreducible components of f (over Q).":

    local j :: satisfies(j1 -> j1 :: list, j1 -> numelems(j1) = 2, j1 -> j1[1] :: polynom, j1 -> j1[2] :: posint):
    # j is just an element in the output list of Maple\'s built-in 'factors' command.

    if integerPolynomials = true then
        return Array([seq(denom(content(j[1])) * j[1], j in factors(f)[2])]): # with integer coefficients only.
    else
        return Array([seq(j[1], j in factors(f)[2])]): # monic polynomials only the same as in the output of factors.
    end if:

end proc: