##MODULE(help) ResChain
##HALFLINE elimination of n variables from m polynomials using iterated univariate resultants.
##AUTHOR AmirHosein Sadeghimanesh
##DATE November 2021
##EXAMPLES
##> with(ResChain);
##> [ResChainBranching, ResChainSimple]
##> F := [(x^2+y^2-a)*(y-x^2), (x+y)*(x*y-b), (x^3+a*y^2+b)]:
##> vars := [y, x]:
##> parameters_conditions_1 := ResChainSimple(F, vars);
##- parameters_conditions_1 := [a, a^3*b + 8, b, -a^6*b^8 + 9*a^5*b^8 - 18*a^4*b^8 + 6*a^3*b^9 - b^12 - 9*a^7*b^4 - 6*a^6*b^5 + 3*a^3*b^8 - 18*a^2*b^9 - 3*a^6*b^4 - 36*a^5*b^5 - 27*a^4*b^6 - 2*a^3*b^7 - 3*b^10 + a^9 - 21*a^3*b^6 - 18*a^2*b^7 - 3*a^6*b^2 - 3*b^8 + 3*a^3*b^4 - b^6, a^3 - b^2, a + b - 1, a + b + 1, a^3 - 4*a^2 - 2*a*b - b^2 + 3*a - 1, a^8*b^4 + 4*a^8*b^3 + 6*a^8*b^2 + 4*a^6*b^4 + 4*a^8*b + 12*a^6*b^3 + a^8 + 12*a^6*b^2 + 6*a^4*b^4 + 4*a^6*b - a^5*b^2 + 12*a^4*b^3 + 6*a^4*b^2 - 8*a^3*b^3 + 4*a^2*b^4 - 6*a^3*b^2 + 4*a^2*b^3 - 4*a^3*b + 8*a*b^3 + b^4 - a^3 + 3*a*b^2 + b^2, -a^4*b^4 + a^7 - 5*a^5*b^2 - 2*a^4*b^3 + 5*a^3*b^4 + 2*a^5*b - a^4*b^2 - 8*a^3*b^3 - 2*a^2*b^4 + 4*a*b^5 - b^6 + a^3*b^2 - 2*a^2*b^3 - 3*a*b^4 - b^4, a^8 + 8*a^6*b + 24*a^4*b^2 + 32*a^2*b^3 + 16*b^4 - 2*a^3, a^4 - 1/2*a^3 + 4*a^2*b + 4*b^2]
##> parameters_conditions_2 := ResChainBranching(F, vars);
##- parameters_conditions_2 := [-a^4*b^4 + a^7 - 5*a^5*b^2 - 2*a^4*b^3 + 5*a^3*b^4 + 2*a^5*b - a^4*b^2 - 8*a^3*b^3 - 2*a^2*b^4 + 4*a*b^5 - b^6 + a^3*b^2 - 2*a^2*b^3 - 3*a*b^4 - b^4, a^4 - 1/2*a^3 + 4*a^2*b + 4*b^2, a, a^3*b + 8, b, a + b - 1, a + b + 1]
##> evalb(`and`(seq(polynomial in parameters_conditions_1, polynomial in parameters_conditions_2)));
##- true
##SEEALSO
##- resultant
##- PolynomialIdeals:-EliminationIdeal

unprotect('ResChain');

ResChain := module()

    option package;

    description "The purpose of this package is to use a chain of resultants to eliminate n variables from m polynomials. In another word, consider a system of m polynomial equations in n variables and some parameters. The goal is to project the solution set of this system from space of both variables and parameters into the space of only parameters. The result is a set of conditions involving only parameters for the system to have a solution. This is somehow similar to elimination theory via Grobner bases, or when m = n + 1, similar to the Dixon resultant. However, the result with a chain of resultants gives a superset of the Zariski closure of the projection. The other two methods (Grobner basis and the Dixon resultant) give the Zariski closure of the projection which is a better set, but they may be slower or more expensive in some cases. Therefore, there is a trade off between the different approaches.";

    local Factorize, ArrayAddIfNotIn;

    export ResChainSimple, ResChainBranching;

# Getting the directory where this file, "main.mpl" is located.
kernelopts(includepath=currentdir()): # For the next lines, all paths are relative to this folder.
# including functions.
## tools for working with polynomials and arrays.
$include "Tools\\Array_Add_If_Not_In.mm"
$include "Tools\\Factorize.mm"
## ResChainSimple
$include "ResChainSimple\\ResChainSimple.mm"
## ResChainBranching
$include "ResChainBranching\\ResChainBranching.mm"


    # end of the package
end module;

protect('ResChain');